@@ -52,6 +52,16 @@ GameArea::~GameArea()
   delete area;
 }
 
+bool GameArea::HandlesClickFocus() const
+{
+  return CanFocus() && IsVisible();
+}
+
+bool GameArea::HandlesKeyFocus() const
+{
+  return CanFocus() && IsVisible();
+}
+
 void GameArea::UpdateDimensions()
 {
   if (area!=NULL) {
@@ -64,99 +74,7 @@ void GameArea::UpdateDimensions()
   }
 }
 
-/**
-  Create bitmap containing the game area background build up from walls, floors,
-  goals and backgrounds.
- */
-void GameArea::RebuildArea()
-{
-  delete area;
-
-  area=NULL;
-
-  if (areaWidth>0 && areaHeight>0) {
-    area=Lum::OS::driver->CreateBitmap(areaWidth,areaHeight);
-  }
-
-  UpdateDimensions(); // TODO: Is this really necessray?
-}
-
-void GameArea::RedrawArea()
-{
-  if (!currentSet.Valid() ||
-      currentSet->IsNull() ||
-      area==NULL) {
-    return;
-  }
-
-  size_t                             x,y,i;
-  std::list<DiceSet>::const_iterator iter=sets.begin();
-  Lum::OS::DrawInfo                  *draw=area->GetDrawInfo();
-
-  i=1;
-  x=distance;
-  y=distance;
-
-  GetBackground()->Draw(draw,
-                        0,0,areaWidth,areaHeight,
-                        0,0,areaWidth,areaHeight);
-
-  if (*currentSet>0) {
-    std::advance(iter,*currentSet-1);
-  }
-
-  for (std::list<Dice>::const_iterator dice=iter->dices.begin();
-       dice!=iter->dices.end();
-       dice++) {
-
-    if (dice->stays) {
-      draw->PushForeground(staysFrameColor);
-    }
-    else {
-      draw->PushForeground(normalFrameColor);
-    }
-
-    draw->PushPen(diceFrame,Lum::OS::DrawInfo::penNormal);
-    draw->DrawRectangle(x,y,cellSize,cellSize);
-    draw->PopForeground();
-    draw->PopPen();
-
-    draw->PushForeground(cellBackgroundColor);
-    draw->FillRectangle(x+diceFrame,y+diceFrame,cellSize-2*diceFrame,cellSize-2*diceFrame);
-    draw->PopForeground();
-
-    if (!dice->value.empty()) {
-      Lum::OS::FontExtent extent;
-
-      font->StringExtent(dice->value,extent,Lum::OS::Font::normal);
-
-      draw->PushFont(font);
-      draw->PushForeground(textColor);
-      draw->DrawString(x-extent.left+(cellSize-(extent.width-extent.left-extent.right))/2,
-                       y+(cellSize-font->height)/2+font->ascent,
-                       dice->value);
-      draw->PopForeground();
-      draw->PopFont();
-    }
-
-    if (i%columns==0) {
-      x=distance;
-      y+=cellSize+distance;
-    }
-    else {
-      x+=cellSize+distance;
-    }
-
-    i++;
-  }
-}
-
-bool GameArea::HandlesFocus() const
-{
-  return RequestedFocus() && visible;
-}
-
-void GameArea::CalcAreaSize()
+void GameArea::CalcAreaSize(size_t width, size_t height)
 {
   if (!currentSet.Valid() ||
       currentSet->IsNull() ||
@@ -272,6 +190,102 @@ void GameArea::CalcAreaSize()
   }
 }
 
+/**
+  Create bitmap containing the game area background build up from walls, floors,
+  goals and backgrounds.
+ */
+void GameArea::RebuildArea()
+{
+  delete area;
+
+  area=NULL;
+
+  if (areaWidth>0 && areaHeight>0) {
+    area=Lum::OS::driver->CreateBitmap(areaWidth,areaHeight);
+  }
+
+  UpdateDimensions();
+}
+
+void GameArea::RedrawArea()
+{
+  if (!currentSet.Valid() ||
+      currentSet->IsNull() ||
+      area==NULL) {
+    return;
+  }
+
+  size_t                             x,y,i;
+  std::list<DiceSet>::const_iterator iter=sets.begin();
+  Lum::OS::DrawInfo                  *draw=area->GetDrawInfo();
+
+  i=1;
+  x=distance;
+  y=distance;
+
+  GetBackground()->Draw(draw,
+                        0,0,areaWidth,areaHeight,
+                        0,0,areaWidth,areaHeight);
+
+  if (*currentSet>0) {
+    std::advance(iter,*currentSet-1);
+  }
+
+  for (std::list<Dice>::const_iterator dice=iter->dices.begin();
+       dice!=iter->dices.end();
+       dice++) {
+
+    if (dice->stays) {
+      draw->PushForeground(staysFrameColor);
+    }
+    else {
+      draw->PushForeground(normalFrameColor);
+    }
+
+    draw->PushPen(diceFrame,Lum::OS::DrawInfo::penNormal);
+    draw->DrawRectangle(x,y,cellSize,cellSize);
+    draw->PopForeground();
+    draw->PopPen();
+
+    draw->PushForeground(cellBackgroundColor);
+    draw->FillRectangle(x+diceFrame,y+diceFrame,cellSize-2*diceFrame,cellSize-2*diceFrame);
+    draw->PopForeground();
+
+    if (!dice->value.empty()) {
+      Lum::OS::FontExtent extent;
+
+      font->StringExtent(dice->value,extent,Lum::OS::Font::normal);
+
+      draw->PushFont(font);
+      draw->PushForeground(textColor);
+      draw->DrawString(x-extent.left+(cellSize-(extent.width-extent.left-extent.right))/2,
+                       y+(cellSize-font->height)/2+font->ascent,
+                       dice->value);
+      draw->PopForeground();
+      draw->PopFont();
+    }
+
+    if (i%columns==0) {
+      x=distance;
+      y+=cellSize+distance;
+    }
+    else {
+      x+=cellSize+distance;
+    }
+
+    i++;
+  }
+}
+
+void GameArea::Rebuild(size_t width, size_t height)
+{
+  CalcAreaSize(width,height);
+  RebuildArea();
+  if (area!=NULL) {
+    RedrawArea();
+  }
+}
+
 void GameArea::CalcSize()
 {
   size_t cellSize=font->height;
@@ -298,12 +312,31 @@ void GameArea::CalcSize()
   minWidth=6*distance+5*cellSize;
   minHeight=3*distance+2*cellSize;
 
+  width=minWidth;
+  height=minHeight;
+
   Scrollable::CalcSize();
 }
 
-void GameArea::Draw(int x, int y, size_t w, size_t h)
+void GameArea::GetDimension(size_t& width, size_t& height)
 {
-  Scrollable::Draw(x,y,w,h);
+  Rebuild(width,height);
+
+  width=areaWidth;
+  height=areaHeight;
+}
+
+void GameArea::Layout()
+{
+  UpdateDimensions();
+
+  Scrollable::Layout();
+}
+
+void GameArea::Draw(Lum::OS::DrawInfo* draw,
+                    int x, int y, size_t w, size_t h)
+{
+  Scrollable::Draw(draw,x,y,w,h);
 
   if (!OIntersect(x,y,w,h)) {
     return;
@@ -315,8 +348,6 @@ void GameArea::Draw(int x, int y, size_t w, size_t h)
     return;
   }
 
-  Lum::OS::DrawInfo *draw=GetDrawInfo();
-
   draw->CopyFromBitmap(area,
                        hAdjustment->GetTop()-1,vAdjustment->GetTop()-1,
                        width,height,
@@ -381,11 +412,7 @@ void GameArea::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
     Redraw();
   }
   else if (model==currentSet || model==setsModel) {
-    CalcAreaSize();
-    RebuildArea();
-    if (area!=NULL) {
-      RedrawArea();
-    }
+    Rebuild(width,height);
     Redraw();
   }
 
